@@ -1,7 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2005, 2007, 2009, 2010 Free Software Foundation, Inc.
+  Copyright (C) 2005, 2007, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -22,15 +23,13 @@
 
 # include <string>
 
-extern "C"
-{
 # ifdef WIN32
 #  include <direct.h>
 # endif
 # ifdef HAVE_DL
 #  include <dlfcn.h>
 # endif
-}
+
 
 using namespace std;
 
@@ -40,10 +39,10 @@ public:
   int validHandle ();
   DynamicLibrary ();
   ~DynamicLibrary ();
-  int loadLibrary (const char* filename, int globally=0);
-  void* getProc (const char*);
+  int loadLibrary (const char *filename, int globally = 0);
+  void *getProc (const char *sym);
   int close ();
-  const char* getFileName (){return fileName.c_str ();}
+  const char *getFileName (){return fileName.c_str ();}
 private:
   string fileName;
   void *handle;
