@@ -1,7 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2006, 2008, 2009, 2010 Free Software Foundation, Inc.
+  Copyright (C) 2006, 2008, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -41,13 +42,15 @@ public:
 
   int isLoaded (){return loaded;}
 
+  void getLoginName (string &username);
+
 private:
   Mutex loadMutex;
   int loadImpl ();
 
 # ifdef WIN32
   string data;
-# elif !GETPWNAM
+# elif !HAVE_GETPWNAM
   HashMap<string, string*> data;
 # endif
   time_t timestamp;
