@@ -1,7 +1,7 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010 Free Software
+  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@ public:
   long getReadHandle ();
   long getWriteHandle ();
   void inverted (Pipe&);
-  virtual int read (char* buffer, u_long len, u_long *nbr);
-  virtual int write (const char* buffer, u_long len, u_long *nbw);
+  virtual int read (char* buffer, size_t len, size_t *nbr);
+  virtual int write (const char* buffer, size_t len, size_t *nbw);
   virtual int close ();
   void closeRead ();
   void closeWrite ();
@@ -45,11 +45,6 @@ public:
   int waitForData (int sec, int usec);
 private:
   bool terminated;
-# ifndef WIN32
   int handles[2];
-# else
-  HANDLE readHandle;
-  HANDLE writeHandle;
-# endif
 };
 #endif
