@@ -1,9 +1,9 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  * strlcpy and strlcat by codingmaster
-  Copyright (C) 2002, 2003, 2004, 2009, 2010 Free Software Foundation,
-  Inc.
+  strlcpy and strlcat by codingmaster
+  Copyright (C) 2002, 2003, 2004, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
   Copyright (C) 2004 by codingmaster
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
