@@ -1,7 +1,7 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+  Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -22,16 +22,12 @@
 # include "myserver.h"
 # include <include/base/socket/socket.h>
 
-
-extern "C"
-{
 # ifdef WIN32
-#  undef AF_UNIX //This shouldn't happen, but better be sure.
+#  undef AF_UNIX /* This shouldn't happen, but better be sure.  */
 # else
+#  include <string.h>
 #  include <sys/un.h>
 # endif
-}
-
 
 class UnixSocket: public Socket
 {
@@ -39,9 +35,9 @@ public:
   UnixSocket ();
   virtual ~UnixSocket ();
   int bind (const char *path);
-  int shutdown ();
+  int shutdown (int how);
   int close ();
-  int connect (const char* path);
+  int connect2 (const char *path);
   Socket* accept ();
   int socket ();
 protected:
@@ -55,12 +51,7 @@ protected:
 
 private:
 # ifdef AF_UNIX
-  void makeAddrInfo (sockaddr_un *info, const char *path)
-  {
-    memset (info, 0, sizeof (struct sockaddr_un));
-    info->sun_family = AF_UNIX;
-    strcpy (info->sun_path, path);
-  }
+  void makeAddrInfo (sockaddr_un *info, const char *path);
 # endif
 };
 
