@@ -1,7 +1,7 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010 Free Software
+  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,14 +31,14 @@ typedef int Handle;
 typedef int FileHandle;
 
 /*!
- *Abstract class to handle virtual data streams.
+  Abstract class to handle virtual data streams.
  */
 class Stream
 {
 public:
-  virtual int read (char* buffer, u_long len, u_long*);
-  virtual int write (const char* buffer, u_long len, u_long*);
-  virtual int flush (u_long*);
+  virtual int read (char *buffer, size_t len, size_t *);
+  virtual int write (const char *buffer, size_t len, size_t *);
+  virtual int flush (size_t *);
   virtual Handle getHandle ();
   virtual int close ();
   Stream ();
