@@ -1,7 +1,7 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2007, 2009, 2010 Free Software
+  Copyright (C) 2002, 2003, 2004, 2007, 2009, 2010, 2011 Free Software
   Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,17 +20,45 @@
 # define HTTP_HEADER_H
 
 # include "myserver.h"
-
 # include <string>
+
+/* Include for SecurityHeaderData.  */
+# include <include/conf/security/security_token.h>
+
 # include <include/base/hash_map/hash_map.h>
 
 using namespace std;
 
-struct HttpHeader
+class HttpHeader : public SecurityHeaderData
 {
-  virtual string* getValue (const char* name, string* out) = 0;
-  virtual string* setValue (const char* name, const char* in) = 0;
+public:
+  struct Entry
+  {
+    string name;
+    string value;
+    Entry ()
+    {
+    }
+
+    Entry (const string &n, const string &v) : name (NULL), value (NULL)
+    {
+      name.assign (n);
+      value.assign (v);
+    }
+
+    ~Entry ()
+    {
+    }
+  };
+
+  void clearValue (const char *name);
+
+  virtual string* getValue (const char *name, string *out);
+  virtual string* setValue (const char *name, const char *in);
   virtual ~HttpHeader (){}
+
+  /* Key are stored lower-case.  */
+  HashMap<string, struct Entry *> other;
 };
 
 #endif
