@@ -1,7 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2005, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
