@@ -1,7 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-  Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+  2011 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -24,6 +24,7 @@
 #include <include/conf/main/xml_main_configuration.h>
 
 #include <include/base/exceptions/checked.h>
+#include <include/base/home_dir/home_dir.h>
 
 #ifdef WIN32
 # include <direct.h>
@@ -66,7 +67,6 @@ void registerSignals ();
 void Sig_Quit (int signal)
 {
   Server::getInstance ()->log ("Exiting...");
-  sync ();
   Server::getInstance ()->stop ();
   registerSignals ();
 }
@@ -305,7 +305,13 @@ int loadConfFileLocation (string &outFile, string fileName, const char *dir)
           return 0;
         }
 
-      outFile = "~/.myserver/" + fileName;
+      HomeDir homeDir;
+      string login;
+      homeDir.getLoginName (login);
+      homeDir.getHomeDir (login, outFile);
+
+      outFile.append ("/.myserver/");
+      outFile.append (fileName);
       if (FilesUtility::nodeExists (outFile))
         return 0;
 
@@ -428,7 +434,7 @@ int main (int argn, char **argv)
   /* If the version flag is up, show the version and exit.  */
   if (input.version)
     {
-      cout << "GNU MyServer " << MYSERVER_VERSION << endl
+      cout << MYSERVER_VERSION << endl
            << "Copyright (C) 2002-2010 Free Software Foundation, Inc." << endl
            << "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>." << endl
            << "This is free software: you are free to change and redistribute it." << endl
@@ -515,6 +521,8 @@ int main (int argn, char **argv)
     }
   catch (...)
     {
+      if (input.useForkServer)
+        Process::getForkServer ()->killServer ();
       return 1;
     };
 
