@@ -1,7 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2005, 2006, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  Copyright (C) 2005, 2006, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -24,7 +24,7 @@
 
 
 /*!
- *Default constructor.
+  Default constructor.
  */
 DynamicHttpCommand::DynamicHttpCommand (string& name)
 {
@@ -32,7 +32,7 @@ DynamicHttpCommand::DynamicHttpCommand (string& name)
 }
 
 /*!
- *Default constructor.
+  Default constructor.
  */
 string DynamicHttpCommand::getName ()
 {
@@ -40,7 +40,7 @@ string DynamicHttpCommand::getName ()
 }
 
 /*!
- *Destroy the object.
+  Destroy the object.
  */
 DynamicHttpCommand::~DynamicHttpCommand ()
 {
