@@ -1,7 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -25,7 +25,7 @@
 #include <cctype>
 
 /*!
- * Class constructor.
+  Class constructor.
  */
 ProtocolsManager::ProtocolsManager ()
 {
@@ -33,7 +33,7 @@ ProtocolsManager::ProtocolsManager ()
 }
 
 /*!
- * Class destructor.
+  Class destructor.
  */
 ProtocolsManager::~ProtocolsManager ()
 {
@@ -41,7 +41,7 @@ ProtocolsManager::~ProtocolsManager ()
 }
 
 /*!
- * Return a protocol by its name.
+  Return a protocol by its name.
  */
 Protocol* ProtocolsManager::getProtocol (string &name)
 {
@@ -49,7 +49,7 @@ Protocol* ProtocolsManager::getProtocol (string &name)
 }
 
 /*!
- * Add a static protocol to the list.
+  Add a static protocol to the list.
  */
 void ProtocolsManager::addProtocol (string &name, Protocol *protocol)
 {
