@@ -1,7 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-  Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+  2011 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -37,12 +37,11 @@
 #include <include/base/crypt/sha1.h>
 
 #include <include/conf/main/xml_main_configuration.h>
+#include <include/conf/security/xml_validator.h>
 
 #include <cstdarg>
 
 
-extern "C"
-{
 #ifdef WIN32
 # include <direct.h>
 #else
@@ -59,7 +58,7 @@ extern "C"
 # endif
 
 #endif
-}
+
 #include <sstream>
 
 Server* Server::instance = NULL;
@@ -78,7 +77,6 @@ Server::Server () : connectionsPool (100),
   serverReady = false;
   throttlingRate = 0;
   path = 0;
-  ipAddresses = 0;
   vhostHandler = NULL;
   purgeThreadsThreshold = 1;
   freeThreads = 0;
@@ -94,7 +92,7 @@ void
 Server::initLogManager ()
 {
   if (!logLocation.size ())
-    logLocation.assign ("console://stdout");
+    logLocation.assign ("console://stderr");
 
   if (!logManager->containsOwner (this))
     {
@@ -104,8 +102,8 @@ Server::initLogManager ()
 }
 
 /*!
- *Reinitialize the configuration paths, setting them to the specified ones.
- *Returns false on error.
+  Reinitialize the configuration paths, setting them to the specified ones.
+  Returns false on error.
  */
 bool Server::resetConfigurationPaths (string &mainConf, string &mimeConf,
 				     string &vhostConf, string &externPath)
@@ -118,7 +116,7 @@ bool Server::resetConfigurationPaths (string &mainConf, string &mimeConf,
 }
 
 /*!
- * Load here all the libraries.
+  Load here all the libraries.
  */
 int Server::loadLibraries ()
 {
@@ -128,7 +126,6 @@ int Server::loadLibraries ()
   XmlParser::startXML ();
   myserver_safetime_init ();
 
-  gnutls_global_init ();
   if (Socket::startupSocketLib () != 0)
     {
       log (MYSERVER_LOG_MSG_ERROR, _("Error loading the socket library"));
@@ -139,7 +136,7 @@ int Server::loadLibraries ()
 }
 
 /*!
- * Destroy the object.
+  Destroy the object.
  */
 Server::~Server ()
 {
@@ -153,9 +150,6 @@ Server::~Server ()
     delete path;
   path = 0;
 
-  if (ipAddresses)
-    delete ipAddresses;
-
   if (logManager)
     delete logManager;
 
@@ -166,7 +160,7 @@ Server::~Server ()
 }
 
 /*!
- *Start the server.
+  Start the server.
  */
 void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
                     string &externPath, MainConfiguration* (*genMainConf)
@@ -174,10 +168,6 @@ void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
 
 {
   int err = 0;
-#ifdef WIN32
-  DWORD eventsCount, cNumRead;
-  INPUT_RECORD irInBuf[128];
-#endif
 
   this->genMainConf = genMainConf;
 
@@ -206,7 +196,8 @@ void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
         return;
       }
 
-    log (MYSERVER_LOG_MSG_INFO, _("Loading server configuration..."));
+    log (MYSERVER_LOG_MSG_INFO, _("Loading server configuration from %s..."),
+         mainConf.c_str ());
 
     if (postLoad ())
       {
@@ -253,7 +244,7 @@ void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
 }
 
 /*!
- * Complete the loading phase.
+  Complete the loading phase.
  */
 int Server::postLoad ()
 {
@@ -267,17 +258,17 @@ int Server::postLoad ()
   log (MYSERVER_LOG_MSG_INFO, _("Host name: %s"), serverName);
 
   /* Find the IP addresses of the local machine.  */
-  if (ipAddresses)
-    delete ipAddresses;
-  ipAddresses = new string ();
-
-  if (Socket::getLocalIPsList (*ipAddresses))
+  try
+    {
+      Socket::getLocalIPsList (ipAddresses);
+    }
+  catch (exception & e)
     {
       log (MYSERVER_LOG_MSG_ERROR, _("Error reading IP list"));
       return -1;
     }
 
-  log (MYSERVER_LOG_MSG_INFO, _("IP: %s"), ipAddresses->c_str ());
+  log (MYSERVER_LOG_MSG_INFO, _("IP: %s"), ipAddresses.c_str ());
 
   log (MYSERVER_LOG_MSG_INFO, _("Detected %i CPUs"), (int) getCPUCount ());
 
@@ -303,7 +294,8 @@ int Server::postLoad ()
     return -1;
 
   /* Load the MIME types.  */
-  log (MYSERVER_LOG_MSG_INFO, _("Loading MIME types..."));
+  log (MYSERVER_LOG_MSG_INFO, _("Loading MIME types from %s..."),
+       mimeConfigurationFile.c_str ());
 
   string handlerStr (getData ("server.mime_handler", "xml"));
   mimeManager.setDefaultHandler (handlerStr);
@@ -342,7 +334,7 @@ int Server::postLoad ()
 }
 
 /*!
- * Reload the virtual hosts configuration.
+  Reload the virtual hosts configuration.
  */
 int Server::loadVHostConf ()
 {
@@ -360,8 +352,14 @@ int Server::loadVHostConf ()
 
   vhostManager.setHandler (vhostHandler);
 
+  const char *vhostLocation = getData ("server.vhost_location",
+                                       vhostConfigurationFile.c_str ());
+
   /* Load the virtual hosts configuration.  */
-  if (vhostHandler->load (vhostConfigurationFile.c_str ()))
+  log (MYSERVER_LOG_MSG_INFO, _("Loading virtual hosts from %s..."),
+       vhostLocation);
+
+  if (vhostHandler->load (vhostLocation))
     {
       log (MYSERVER_LOG_MSG_ERROR,
            _("Error loading the vhost configuration file %s"),
@@ -373,12 +371,13 @@ int Server::loadVHostConf ()
 }
 
 /*!
- * Load static built-in components.
+  Load static built-in components.
  */
 void Server::loadStaticComponents ()
 {
   string xml ("xml");
 
+  mimeManager.setLogger (this);
   mimeManager.registerHandler (xml, new XmlMimeHandler ());
 
   XmlVhostHandler::registerBuilder (vhostManager);
@@ -411,7 +410,7 @@ void Server::loadStaticComponents ()
 }
 
 /*!
- * Load the plugins.
+  Load the plugins.
  */
 void Server::loadPlugins ()
 {
@@ -421,7 +420,7 @@ void Server::loadPlugins ()
 }
 
 /*!
- * Server main loop.
+  Server main loop.
  */
 void Server::mainLoop ()
 {
@@ -436,16 +435,15 @@ void Server::mainLoop ()
   mimeConfTime = FilesUtility::getLastModTime (mimeConfigurationFile.c_str ());
 
   /*
-   * Keep thread alive.
-   * When the endServer flag is set to True exit
-   * from the loop and terminate the server execution.
+    Keep thread alive.
+    When the endServer flag is set to True exit
+    from the loop and terminate the server execution.
    */
   while (!endServer)
     {
-      Thread::wait (1000000);
+      Thread::wait (MYSERVER_SEC (1));
 
-      /* Check threads.  */
-      if (purgeThreadsCounter++ >= 10)
+      if (purgeThreadsCounter++ >= 120)
         {
           purgeThreadsCounter = 0;
           purgeThreads ();
@@ -453,6 +451,8 @@ void Server::mainLoop ()
 
       if (autoRebootEnabled)
         {
+          bool doBeep = false;
+
           time_t mainConfTimeNow =
             FilesUtility::getLastModTime (mainConfigurationFile.c_str ());
           time_t hostsConfTimeNow =
@@ -461,10 +461,10 @@ void Server::mainLoop ()
             FilesUtility::getLastModTime (mimeConfigurationFile.c_str ());
 
           /* If a configuration file was modified reboot the server. */
-          if (((mainConfTimeNow != -1) && (hostsConfTimeNow != -1)  &&
-               (mimeConfNow != -1)) || toReboot)
+          if (((mainConfTimeNow != -1) && (hostsConfTimeNow != -1)
+               && (mimeConfNow != -1)) || toReboot)
             {
-              if ( (mainConfTimeNow  != mainConfTime) || toReboot)
+              if ((mainConfTimeNow  != mainConfTime) || toReboot)
                 {
                   string msg ("main-conf-changed");
                   notifyMulticast (msg, 0);
@@ -479,13 +479,8 @@ void Server::mainLoop ()
                   string msg ("mime-conf-changed");
                   notifyMulticast (msg, 0);
 
-                  if (logLocation.find ("console://") != string::npos)
-                    {
-                      char beep[] = { static_cast<char>(0x7), '\0' };
-                      string beepStr (beep);
-                      logManager->log (this, "MAINLOG", logLocation, beepStr);
-                    }
-                  log (MYSERVER_LOG_MSG_INFO, _("Reloading MIME types"));
+                  log (MYSERVER_LOG_MSG_INFO, _("Reloading MIME types from %s"),
+                       mimeConfigurationFile.c_str ());
 
                   getMimeManager ()->reload ();
 
@@ -530,13 +525,20 @@ void Server::mainLoop ()
                   hostsConfTime = hostsConfTimeNow;
                   log (MYSERVER_LOG_MSG_INFO, _("Reloaded"));
                 }
+
+              if (doBeep && logLocation.find ("console://") != string::npos)
+                {
+                  char beep[] = { static_cast<char>(0x7), '\0' };
+                  string beepStr (beep);
+                  logManager->log (this, "MAINLOG", logLocation, beepStr);
+                }
             }
         }
     }
 }
 
 /*!
- * Display the MyServer boot.
+  Display the MyServer boot.
  */
 void Server::displayBoot ()
 {
@@ -556,15 +558,12 @@ void Server::displayBoot ()
 
 #endif /* CLEAR_BOOT_SCREEN.  */
 
-  /*
-   * Print the MyServer signature only if the log writes to the console.
-   */
+  /* Print the MyServer signature only if the log writes to the console.  */
   if (logLocation.find ("console://") != string::npos)
     {
       try
         {
-          string softwareSignature = "************ GNU MyServer "
-            MYSERVER_VERSION " ************";
+          string softwareSignature = "************ " MYSERVER_VERSION " ************";
 
           string tmp (softwareSignature.length (), '*');
 
@@ -589,15 +588,15 @@ void Server::displayBoot ()
 }
 
 /*!
- * Removed threads that can be destroyed.
- * The function returns the number of threads that were destroyed.
+  Removed threads that can be destroyed.
+  The function returns the number of threads that were destroyed.
  */
 int Server::purgeThreads ()
 {
   u_long ticks = getTicks ();
   u_long destroyed = 0;
 
-  purgeThreadsThreshold = std::min (purgeThreadsThreshold << 1,
+  purgeThreadsThreshold = std::min (purgeThreadsThreshold * 3 / 2,
                                     nMaxThreads);
   threadsMutex->lock ();
   for (list<ClientsThread*>::iterator it = threads.begin ();
@@ -605,21 +604,14 @@ int Server::purgeThreads ()
     {
       ClientsThread* thread = *it;
 
-
-      /*
-       *Shutdown all threads that can be destroyed.
-       */
       if (thread->isStopped ())
         {
           ClientsThread* thread = *it;
           list<ClientsThread*>::iterator next = it;
           next++;
 
-          thread->join ();
           threads.erase (it);
           delete thread;
-
-          destroyed++;
           it = next;
         }
       else if (thread->isToDestroy ())
@@ -627,14 +619,17 @@ int Server::purgeThreads ()
           if (destroyed < purgeThreadsThreshold)
             thread->stop ();
 
+          destroyed++;
           it++;
         }
       else
         {
-          if (!thread->isStatic ())
-            if (ticks - thread->getTimeout () > MYSERVER_SEC (15))
-              thread->setToDestroy (1);
-
+          if (! thread->isStatic ())
+            if (ticks - thread->getTimeout () > MYSERVER_SEC (20))
+              {
+                thread->setToDestroy (1);
+                destroyed++;
+              }
           it++;
         }
     }
@@ -644,7 +639,7 @@ int Server::purgeThreads ()
 }
 
 /*!
- * Do the final cleanup.  Called once when the process is terminated.
+  Do the final cleanup.  Called once when the process is terminated.
  */
 void Server::finalCleanup ()
 {
@@ -654,7 +649,7 @@ void Server::finalCleanup ()
 }
 
 /*!
- * Return the user identifier to use for the process.
+  Return the user identifier to use for the process.
  */
 const char *Server::getUid ()
 {
@@ -662,7 +657,7 @@ const char *Server::getUid ()
 }
 
 /*!
- * Return the group identifier to use for the process.
+  Return the group identifier to use for the process.
  */
 const char *Server::getGid ()
 {
@@ -670,7 +665,7 @@ const char *Server::getGid ()
 }
 
 /*!
- * Returns the numbers of active connections the list.
+  Returns the numbers of active connections the list.
  */
 u_long Server::getNumConnections ()
 {
@@ -678,7 +673,7 @@ u_long Server::getNumConnections ()
 }
 
 /*!
- * Returns the numbers of all the connections to the server.
+  Returns the numbers of all the connections to the server.
  */
 u_long Server::getNumTotalConnections ()
 {
@@ -686,7 +681,7 @@ u_long Server::getNumTotalConnections ()
 }
 
 /*!
- * Return a home directory object.
+  Return a home directory object.
  */
 HomeDir* Server::getHomeDir ()
 {
@@ -694,7 +689,7 @@ HomeDir* Server::getHomeDir ()
 }
 
 /*!
- * Stop the execution of the server.
+  Stop the execution of the server.
  */
 void Server::stop ()
 {
@@ -702,8 +697,8 @@ void Server::stop ()
 }
 
 /*!
- * Unload the server.
- * Return nonzero on errors.
+  Unload the server.
+  Return nonzero on errors.
  */
 int Server::terminate ()
 {
@@ -739,15 +734,13 @@ int Server::terminate ()
 
   freeHashedData ();
 
-  ipAddresses = NULL;
-
   if (vhostHandler)
     delete vhostHandler;
   vhostHandler = NULL;
 
 #ifdef WIN32
   /*
-   * Under WIN32 cleanup environment strings.
+    Under WIN32 cleanup environment strings.
    */
   FreeEnvironmentStrings ((LPTSTR)envString);
 #endif
@@ -763,7 +756,7 @@ int Server::terminate ()
   clearMulticastRegistry ();
 
   /*
-   * Free all the threads.
+    Free all the threads.
    */
   threadsMutex->lock ();
   threads.clear ();
@@ -780,8 +773,8 @@ int Server::terminate ()
 }
 
 /*!
- * Get a pointer to the configuration file.  It is
- * valid only at startup!
+  Get a pointer to the configuration file.  It is
+  valid only at startup!
  */
 MainConfiguration *Server::getConfiguration ()
 {
@@ -789,9 +782,9 @@ MainConfiguration *Server::getConfiguration ()
 }
 
 /*!
- * Here is loaded the configuration of the server.
- * The configuration file is a XML file.
- * Return nonzero on errors.
+  Here is loaded the configuration of the server.
+  The configuration file is a XML file.
+  Return nonzero on errors.
  */
 int Server::initialize ()
 {
@@ -843,9 +836,7 @@ int Server::initialize ()
 
   configurationFileManager->readData (&hashedDataTrees, &hashedData);
 
-  /*
-   * Process console colors information.
-   */
+  /* Process console colors information.  */
   list<string> levels = logManager->getLoggingLevelsByNames ();
   for (list<string>::iterator it = levels.begin (); it != levels.end (); it++)
     {
@@ -865,11 +856,11 @@ int Server::initialize ()
 
   initLogManager ();
 
-  data = getData ("server.buffer_size");
+  data = getData ("server.buffer_size", "102400");
   if (data)
     buffersize = (atol (data) > 81920) ?  atol (data) :  81920 ;
 
-  data = getData ("server.connection_timeout");
+  data = getData ("connection.timeout");
   if (data)
     connectionTimeout = MYSERVER_SEC ((u_long) atol (data));
 
@@ -960,16 +951,16 @@ int Server::initialize ()
 }
 
 /*!
- * Check if there are free threads to handle a new request.  If there
- * are not enough threads create a new one.
+  Check if there are free threads to handle a new request.  If there
+  are not enough threads create a new one.
  */
 void Server::checkThreadsNumber ()
 {
   threadsMutex->lock ();
 
   /*
-   *Create a new thread if there are not available threads and
-   *we did not reach the limit.
+    Create a new thread if there are not available threads and
+    we did not reach the limit.
    */
   if ((threads.size () < nMaxThreads) && (freeThreads < 1))
     addThread (false);
@@ -978,7 +969,7 @@ void Server::checkThreadsNumber ()
 }
 
 /*!
- * Get the default throttling rate to use with connections to the server.
+  Get the default throttling rate to use with connections to the server.
  */
 u_long Server::getThrottlingRate ()
 {
@@ -986,8 +977,8 @@ u_long Server::getThrottlingRate ()
 }
 
 /*!
- * This function returns the max size of the logs file as defined in the
- * configuration file.
+  This function returns the max size of the logs file as defined in the
+  configuration file.
  */
 int Server::getMaxLogFileSize ()
 {
@@ -995,7 +986,7 @@ int Server::getMaxLogFileSize ()
 }
 
 /*!
- * Returns the connection timeout.
+  Returns the connection timeout.
  */
 u_long Server::getTimeout ()
 {
@@ -1003,10 +994,10 @@ u_long Server::getTimeout ()
 }
 
 /*!
- * This function add a new connection to the list.
- * On a failure, it is responsibility of the caller to
- * free S.
- * \return 0 on success.
+  This function add a new connection to the list.
+  On a failure, it is responsibility of the caller to
+  free S.
+  \return 0 on success.
  */
 int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
 {
@@ -1018,13 +1009,13 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
   int dim;
 
   /*
-   * We can use MAX_IP_STRING_LEN only because we use NI_NUMERICHOST
-   * in getnameinfo call; Otherwise we should have used NI_MAXHOST.
-   * ip is the string containing the address of the remote host connecting
-   * to the server.
-   * localIp is the local address used by the connection.
-   * port is the remote port used by the client to open the connection.
-   * myPort is the port used by the server to listen.
+    We can use MAX_IP_STRING_LEN only because we use NI_NUMERICHOST
+    in getnameinfo call; Otherwise we should have used NI_MAXHOST.
+    ip is the string containing the address of the remote host connecting
+    to the server.
+    localIp is the local address used by the connection.
+    port is the remote port used by the client to open the connection.
+    myPort is the port used by the server to listen.
    */
   if ( asockIn == NULL ||
        (asockIn->ss_family != AF_INET && asockIn->ss_family != AF_INET6))
@@ -1037,22 +1028,23 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
     return -1;
 
   /*
-   * Do not accept this connection if a MAX_CONNECTIONS_TO_ACCEPT limit is
-   * defined.
+    Do not accept this connection if a MAX_CONNECTIONS_TO_ACCEPT limit is
+    defined.
    */
   if (maxConnectionsToAccept && connectionsScheduler.getNumAliveConnections ()
       >= maxConnectionsToAccept)
     return -1;
 
 #if ( HAVE_IPV6 )
+  int ret;
   if ( asockIn->ss_family == AF_INET )
-    ret = getnameinfo (reinterpret_cast<const sockaddr *>(asockIn),
-                       sizeof (sockaddr_in),
-                       ip, MAX_IP_STRING_LEN, NULL, 0, NI_NUMERICHOST);
+    ret = gnulib::getnameinfo (reinterpret_cast<const sockaddr *>(asockIn),
+                               sizeof (sockaddr_in),
+                               ip, MAX_IP_STRING_LEN, NULL, 0, NI_NUMERICHOST);
   else
-    ret = getnameinfo (reinterpret_cast<const sockaddr *>(asockIn),
-                       sizeof (sockaddr_in6),  ip, MAX_IP_STRING_LEN,
-                       NULL, 0, NI_NUMERICHOST);
+    ret = gnulib::getnameinfo (reinterpret_cast<const sockaddr *>(asockIn),
+                               sizeof (sockaddr_in6),  ip, MAX_IP_STRING_LEN,
+                               NULL, 0, NI_NUMERICHOST);
   if (ret)
     return -1;
 
@@ -1063,23 +1055,23 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
   s->getsockname ((MYSERVER_SOCKADDR*)&localSockIn, &dim);
 
   if ( asockIn->ss_family == AF_INET )
-    ret = getnameinfo (reinterpret_cast<const sockaddr *>(&localSockIn),
-                       sizeof (sockaddr_in), localIp, MAX_IP_STRING_LEN,
-                       NULL, 0, NI_NUMERICHOST);
-  else// AF_INET6
-    ret = getnameinfo (reinterpret_cast<const sockaddr *>(&localSockIn),
-                       sizeof (sockaddr_in6), localIp, MAX_IP_STRING_LEN,
-                       NULL, 0, NI_NUMERICHOST);
+    ret = gnulib::getnameinfo (reinterpret_cast<const sockaddr *>(&localSockIn),
+                               sizeof (sockaddr_in), localIp, MAX_IP_STRING_LEN,
+                               NULL, 0, NI_NUMERICHOST);
+  else
+    ret = gnulib::getnameinfo (reinterpret_cast<const sockaddr *>(&localSockIn),
+                               sizeof (sockaddr_in6), localIp, MAX_IP_STRING_LEN,
+                               NULL, 0, NI_NUMERICHOST);
   if (ret)
     return -1;
-#else// !HAVE_IPV6
+#else
   dim = sizeof (localSockIn);
   s->getsockname ((MYSERVER_SOCKADDR*)&localSockIn, &dim);
   strncpy (ip,  inet_ntoa (((sockaddr_in *)asockIn)->sin_addr),
            MAX_IP_STRING_LEN);
   strncpy (localIp, inet_ntoa (((sockaddr_in *)&localSockIn)->sin_addr),
            MAX_IP_STRING_LEN);
-#endif//HAVE_IPV6
+#endif
 
   /* Port used by the client.  */
   if ( asockIn->ss_family == AF_INET )
@@ -1101,7 +1093,7 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
 }
 
 /*!
- * Return the max number of threads that the server can start.
+  Return the max number of threads that the server can start.
  */
 int Server::getMaxThreads ()
 {
@@ -1109,8 +1101,8 @@ int Server::getMaxThreads ()
 }
 
 /*!
- * Add a new connection.
- *A connection is defined using a connection struct.
+  Add a new connection.
+  A connection is defined using a connection struct.
  */
 ConnectionPtr Server::addConnectionToList (Socket* s,
                                            MYSERVER_SOCKADDRIN* /*asockIn*/,
@@ -1126,8 +1118,16 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
   vector<Multicast<string, void*, int>*>* handlers;
 
   connectionsPoolLock.lock ();
-  newConnection = connectionsPool.forcedGet ();
-  connectionsPoolLock.unlock ();
+  try
+    {
+      newConnection = connectionsPool.forcedGet ();
+      connectionsPoolLock.unlock ();
+    }
+  catch (...)
+    {
+      connectionsPoolLock.unlock ();
+      throw;
+    }
 
   if (!newConnection)
     return NULL;
@@ -1142,9 +1142,17 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
   newConnection->host = vhostManager.getVHost (0, localIpAddr, localPort);
   if (newConnection->host == NULL)
     {
-      connectionsPoolLock.lock ();
-      connectionsPool.put (newConnection);
-      connectionsPoolLock.unlock ();
+      try
+        {
+          connectionsPoolLock.lock ();
+          connectionsPool.put (newConnection);
+          connectionsPoolLock.unlock ();
+        }
+      catch (...)
+        {
+          connectionsPoolLock.unlock ();
+          throw;
+        }
       return 0;
     }
 
@@ -1166,8 +1174,17 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
         if ((*handlers)[i]->updateMulticast (this, msg, newConnection) == 1)
           {
             connectionsPoolLock.lock ();
-            connectionsPool.put (newConnection);
-            connectionsPoolLock.unlock ();
+            try
+              {
+                connectionsPool.put (newConnection);
+                connectionsPoolLock.unlock ();
+              }
+            catch (...)
+              {
+                connectionsPoolLock.unlock ();
+                throw;
+              }
+
             return 0;
           }
     }
@@ -1176,17 +1193,26 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
   if (doSSLhandshake)
     {
       int ret = 0;
-      SSL_CTX* ctx = newConnection->host->getSSLContext ();
       SslSocket *sslSocket = new SslSocket (s);
 
-      sslSocket->setSSLContext (ctx);
-      ret = sslSocket->sslAccept ();
+      sslSocket->setSSLContext (newConnection->host->getSSLContext (),
+                                newConnection->host->getSSLPriorityCache ());
 
+      ret = sslSocket->sslAccept ();
       if (ret < 0)
         {
           connectionsPoolLock.lock ();
-          connectionsPool.put (newConnection);
-          connectionsPoolLock.unlock ();
+          try
+            {
+              connectionsPool.put (newConnection);
+              connectionsPoolLock.unlock ();
+            }
+          catch (...)
+            {
+              connectionsPoolLock.unlock ();
+              throw;
+            }
+
           delete sslSocket;
           return 0;
         }
@@ -1206,9 +1232,9 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
     connectionsScheduler.addNewWaitingConnection (newConnection);
 
   /*
-   * If defined maxConnections and the number of active connections
-   * is bigger than it say to the protocol that will parse the connection
-   * to remove it from the active connections list.
+    If defined maxConnections and the number of active connections
+    is bigger than it say to the protocol that will parse the connection
+    to remove it from the active connections list.
    */
   if (maxConnections
       && connectionsScheduler.getNumAliveConnections () > maxConnections)
@@ -1220,7 +1246,7 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
 }
 
 /*!
- * Delete a connection.
+  Delete a connection.
  */
 int Server::deleteConnection (ConnectionPtr s)
 {
@@ -1230,14 +1256,22 @@ int Server::deleteConnection (ConnectionPtr s)
   s->destroy ();
 
   connectionsPoolLock.lock ();
-  connectionsPool.put (s);
-  connectionsPoolLock.unlock ();
+  try
+    {
+      connectionsPool.put (s);
+      connectionsPoolLock.unlock ();
+    }
+  catch (...)
+    {
+      connectionsPoolLock.unlock ();
+      throw;
+    }
 
   return 0;
 }
 
 /*!
- * Get notified when a connection is closed.
+  Get notified when a connection is closed.
  */
 int Server::notifyDeleteConnection (ConnectionPtr s)
 {
@@ -1254,7 +1288,7 @@ int Server::notifyDeleteConnection (ConnectionPtr s)
 }
 
 /*!
- * Get a connection to parse.
+  Get a connection to parse.
  */
 ConnectionPtr Server::getConnection (int /*id*/)
 {
@@ -1262,7 +1296,7 @@ ConnectionPtr Server::getConnection (int /*id*/)
 }
 
 /*!
- * Delete all the active connections.
+  Delete all the active connections.
  */
 void Server::clearAllConnections ()
 {
@@ -1284,8 +1318,8 @@ void Server::clearAllConnections ()
 
 
 /*!
- * Returns the full path of the binaries directory.
- * The directory where the file myserver (.exe) is.
+  Returns the full path of the binaries directory.
+  The directory where the file myserver (.exe) is.
  */
 const char *Server::getPath ()
 {
@@ -1293,7 +1327,7 @@ const char *Server::getPath ()
 }
 
 /*!
- * Add a free thread.
+  Add a free thread.
  */
 void Server::increaseFreeThread ()
 {
@@ -1303,7 +1337,7 @@ void Server::increaseFreeThread ()
 }
 
 /*!
- * Remove a free thread.
+  Remove a free thread.
  */
 void Server::decreaseFreeThread ()
 {
@@ -1314,7 +1348,7 @@ void Server::decreaseFreeThread ()
 
 
 /*!
- * Returns the name of the server (the name of the current PC).
+  Returns the name of the server (the name of the current PC).
  */
 const char *Server::getServerName ()
 {
@@ -1322,29 +1356,37 @@ const char *Server::getServerName ()
 }
 
 /*!
- * Gets the number of threads.
- */
-u_long Server::getNumThreads ()
+  Get Information on the number of threads.
+  \param num if not NULL will contain the number of active threads.
+  \param max if not NULL will contain the maximum number of threads.
+  \param staticThreads if not NULL will contain the maximum number of always
+  active threads.
+*/
+void Server::getThreadsNumberInformation (u_long *num, u_long *max,
+                                          u_long *staticThreads)
 {
-  u_long ret;
-  threadsMutex->lock ();
-  ret = threads.size ();
-  threadsMutex->unlock ();
-  return ret;
+  if (num)
+    *num = threads.size ();
+
+  if (max)
+    *max = nMaxThreads;
+
+  if (staticThreads)
+    *staticThreads = nStaticThreads;
 }
 
 /*!
- * Returns a comma-separated local machine IPs list.
- * For example: 192.168.0.1, 61.62.63.64, 65.66.67.68.69
+  Returns a comma-separated local machine IPs list.
+  For example: 192.168.0.1, 61.62.63.64, 65.66.67.68.69
  */
 const char *Server::getAddresses ()
 {
-  return ipAddresses ? ipAddresses->c_str () : "";
+  return ipAddresses.c_str ();
 }
 
 /*!
- * Clear the data dictionary.
- * Returns zero on success.
+  Clear the data dictionary.
+  Returns zero on success.
  */
 int Server::freeHashedData ()
 {
@@ -1368,8 +1410,8 @@ int Server::freeHashedData ()
 }
 
 /*!
- * Get the value for name in the hash dictionary.
- * If the value is not found then defValue is returned.
+  Get the value for name in the hash dictionary.
+  If the value is not found then defValue is returned.
  */
 const char *Server::getData (const char *name, const char *defValue)
 {
@@ -1379,7 +1421,7 @@ const char *Server::getData (const char *name, const char *defValue)
 }
 
 /*!
- * Get the specified protocol.
+  Get the specified protocol.
  */
 Protocol* Server::getProtocol (const char *protocolName)
 {
@@ -1389,13 +1431,13 @@ Protocol* Server::getProtocol (const char *protocolName)
 }
 
 /*!
- * If specified set the uid/gid for the process.
+  If specified set the uid/gid for the process.
  */
 void Server::setProcessPermissions ()
 {
   /*
-   * If the configuration specify a group id, change the current group for
-   * the process.
+    If the configuration specify a group id, change the current group for
+    the process.
    */
   if (gid.length ())
     {
@@ -1411,9 +1453,9 @@ void Server::setProcessPermissions ()
     }
 
   /*
-   * If the configuration file provides a user identifier, change the
-   * current user for the process. Disable the reboot when this feature
-   * is used.
+    If the configuration file provides a user identifier, change the
+    current user for the process. Disable the reboot when this feature
+    is used.
    */
   if (uid.length ())
     {
@@ -1426,8 +1468,8 @@ void Server::setProcessPermissions ()
 }
 
 /*!
- * Reboot the server.
- * Returns non zero on errors.
+  Reboot the server.
+  Returns non zero on errors.
  */
 int Server::reboot ()
 {
@@ -1493,7 +1535,7 @@ int Server::reboot ()
 }
 
 /*!
- * Returns if the server is ready.
+  Returns if the server is ready.
  */
 bool Server::isServerReady ()
 {
@@ -1501,7 +1543,7 @@ bool Server::isServerReady ()
 }
 
 /*!
- * Reboot the server on the next loop.
+  Reboot the server on the next loop.
  */
 void Server::delayedReboot ()
 {
@@ -1510,7 +1552,7 @@ void Server::delayedReboot ()
 }
 
 /*!
- * Return the factory object to create cached files.
+  Return the factory object to create cached files.
  */
 CachedFileFactory* Server::getCachedFiles ()
 {
@@ -1518,7 +1560,7 @@ CachedFileFactory* Server::getCachedFiles ()
 }
 
 /*!
- * Get a list with all the alive connections.
+  Get a list with all the alive connections.
  */
 void Server::getConnections (list<ConnectionPtr>& out)
 {
@@ -1526,7 +1568,7 @@ void Server::getConnections (list<ConnectionPtr>& out)
 }
 
 /*!
- * Disable the autoreboot.
+  Disable the autoreboot.
  */
 void Server::disableAutoReboot ()
 {
@@ -1534,7 +1576,7 @@ void Server::disableAutoReboot ()
 }
 
 /*!
- * Enable the autoreboot
+  Enable the autoreboot
  */
 void Server::enableAutoReboot ()
 {
@@ -1542,7 +1584,7 @@ void Server::enableAutoReboot ()
 }
 
 /*!
- * Return the ProtocolManager object.
+  Return the ProtocolManager object.
  */
 ProtocolsManager *Server::getProtocolsManager ()
 {
@@ -1550,7 +1592,7 @@ ProtocolsManager *Server::getProtocolsManager ()
 }
 
 /*!
- * Return nonzero if the autoreboot is enabled.
+  Return nonzero if the autoreboot is enabled.
  */
 bool Server::isAutorebootEnabled ()
 {
@@ -1558,7 +1600,7 @@ bool Server::isAutorebootEnabled ()
 }
 
 /*!
- * Create a new thread.
+  Create a new thread.
  */
 int Server::addThread (bool staticThread)
 {
@@ -1595,7 +1637,6 @@ int Server::addThread (bool staticThread)
   newThread = new ClientsThread (this);
 
   handlers = getHandlers (msg);
-
   if (handlers)
     {
       for (size_t i = 0; i < handlers->size (); i++)
@@ -1621,8 +1662,8 @@ int Server::addThread (bool staticThread)
 }
 
 /*!
- * Remove a thread.
- * Return zero if a thread was removed.
+  Remove a thread.
+  Return zero if a thread was removed.
  */
 int Server::removeThread (u_long ID)
 {
@@ -1659,9 +1700,9 @@ int Server::removeThread (u_long ID)
 }
 
 /*!
- * Create the class instance.  Call this before use
- * the Server class.  The instance is not created in
- * getInstance to have a faster inline function.
+  Create the class instance.  Call this before use
+  the Server class.  The instance is not created in
+  getInstance to have a faster inline function.
  */
 void Server::createInstance ()
 {
@@ -1670,7 +1711,7 @@ void Server::createInstance ()
 }
 
 /*!
- * Get a pointer to a filters factory object.
+  Get a pointer to a filters factory object.
  */
 FiltersFactory* Server::getFiltersFactory ()
 {
@@ -1678,7 +1719,7 @@ FiltersFactory* Server::getFiltersFactory ()
 }
 
 /*!
- * Check how many threads are not working.
+  Check how many threads are not working.
  */
 int Server::countAvailableThreads ()
 {
@@ -1694,10 +1735,10 @@ int Server::countAvailableThreads ()
 }
 
 /*!
- * Write a message on a single LogStream specifying a formatting string.
- *
- * \see LogManager#log (void*, string, string, LoggingLeve, bool, va_list)
- * \return 0 on success, 1 on error.
+  Write a message on a single LogStream specifying a formatting string.
+
+  \see LogManager#log (void*, string, string, LoggingLeve, bool, va_list)
+  \return 0 on success, 1 on error.
  */
 int Server::log (LoggingLevel level, const char *fmt, ...)
 {
@@ -1718,7 +1759,7 @@ int Server::log (LoggingLevel level, const char *fmt, ...)
 }
 
 /*!
- * Write a string to the log file and terminate the line.
+  Write a string to the log file and terminate the line.
  */
 int Server::log (char const* str, LoggingLevel level)
 {
@@ -1726,7 +1767,7 @@ int Server::log (char const* str, LoggingLevel level)
 }
 
 /*!
- * Use a specified location as log.
+  Use a specified location as log.
  */
 int
 Server::setLogLocation (string location)
@@ -1742,7 +1783,7 @@ Server::setLogLocation (string location)
 }
 
 /*!
- * Get the size for the first buffer.
+  Get the size for the first buffer.
  */
 u_long Server::getBuffersize ()
 {
